"""
GSIServer - Contributed by Ivan R. Judson <judson@mcs.anl.gov>


################################################################################
#
# SOAPpy - Cayce Ullman       (cayce@actzero.com)
#          Brian Matthews     (blm@actzero.com)
#          Gregory Warnes     (gregory_r_warnes@groton.pfizer.com)
#          Christopher Blunck (blunck@gst.com)
#
################################################################################
# Copyright (c) 2003, Pfizer
# Copyright (c) 2001, Cayce Ullman.
# Copyright (c) 2001, Brian Matthews.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# Neither the name of actzero, inc. nor the names of its contributors may
# be used to endorse or promote products derived from this software without
# specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################
"""

ident = '$Id: GSIServer.py,v 1.4 2004/02/04 03:21:02 irjudson Exp $'
from version import __version__

from __future__ import nested_scopes

#import xml.sax
import re
import socket
import sys
import SocketServer
from types import *
import BaseHTTPServer

# SOAPpy modules
from Parser      import parseSOAPRPC
from Config      import SOAPConfig
from Types       import faultType, voidType, simplify
from NS          import NS
from SOAPBuilder import buildSOAP
from Utilities   import debugHeader, debugFooter

try: from M2Crypto import SSL
except: pass

#####

from Server import *

from pyGlobus.io import GSITCPSocketServer, ThreadingGSITCPSocketServer
from pyGlobus import ioc

def GSIConfig():
    config = SOAPConfig()
    config.channel_mode = ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP
    config.delegation_mode = ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY
    config.tcpAttr = None
    config.authMethod = "_authorize"
    return config

Config = GSIConfig()

class GSISOAPServer(GSITCPSocketServer, SOAPServerBase):
    def __init__(self, addr = ('localhost', 8000),
                 RequestHandler = SOAPRequestHandler, log = 0,
                 encoding = 'UTF-8', config = Config, namespace = None):

        # Test the encoding, raising an exception if it's not known
        if encoding != None:
            ''.encode(encoding)

        self.namespace          = namespace
        self.objmap             = {}
        self.funcmap            = {}
        self.encoding           = encoding
        self.config             = config
        self.log                = log
        
        self.allow_reuse_address= 1
        
        GSITCPSocketServer.__init__(self, addr, RequestHandler,
                                    self.config.channel_mode,
                                    self.config.delegation_mode,
                                    tcpAttr = self.config.tcpAttr)
        
    def get_request(self):
        sock, addr = GSITCPSocketServer.get_request(self)

        return sock, addr
       
class ThreadingGSISOAPServer(ThreadingGSITCPSocketServer, SOAPServerBase):

    def __init__(self, addr = ('localhost', 8000),
                 RequestHandler = SOAPRequestHandler, log = 0,
                 encoding = 'UTF-8', config = Config, namespace = None):
        
        # Test the encoding, raising an exception if it's not known
        if encoding != None:
            ''.encode(encoding)

        self.namespace          = namespace
        self.objmap             = {}
        self.funcmap            = {}
        self.encoding           = encoding
        self.config             = config
        self.log                = log
        
        self.allow_reuse_address= 1
        
        ThreadingGSITCPSocketServer.__init__(self, addr, RequestHandler,
                                             self.config.channel_mode,
                                             self.config.delegation_mode,
                                             tcpAttr = self.config.tcpAttr)

    def get_request(self):
        sock, addr = ThreadingGSITCPSocketServer.get_request(self)

        return sock, addr




#!/usr/bin/env python
#
# $Id: setup.py,v 1.10 2003/05/21 14:51:30 warnes Exp $

CVS=0

from distutils.core import setup, Command, Extension
from SOAPpy.version import __version__

url="http://pywebsvcs.sf.net/"

long_description="SOAPpy provides tools for building SOAP clients and servers.  For more information see " + url


if CVS:
    import time
    __version__ += "_CVS_"  + time.strftime('%Y_%m_%d')


setup(name="SOAPpy",
      version=__version__,
      description="SOAP Services for Python",
      maintainer="Gregory Warnes",
      maintainer_email="gregory_r_warnes@groton.pfizer.com",
      url = url,
      long_description=long_description,
      packages=['SOAPpy','SOAPpy/wstools']
     )




#!/usr/bin/env python

# Copyright (c) 2001 actzero, inc. All rights reserved.

import sys
sys.path.insert(1, "..")

from SOAPpy import *

# Uncomment to see outgoing HTTP headers and SOAP and incoming 
#Config.debug = 1
#Config.dumpHeadersIn = 1
#Config.dumpSOAPIn = 1
#Config.dumpSOAPOut = 1

# ask for returned SOAP responses to be converted to basic python types
Config.simplify_objects = 1

#Config.BuildWithNoType = 1
#Config.BuildWithNoNamespacePrefix = 1

if len(sys.argv) > 1 and sys.argv[1] == '-s':
    # Use secure http
    pathserver = SOAPProxy("https://localhost:9900/pathtest")
    server = SOAPProxy("https://localhost:9900")
    
elif len(sys.argv) > 1 and sys.argv[1] == '-g':
    # use Globus for communication
    import pyGlobus 
    pathserver = SOAPProxy("httpg://localhost:9900/pathtest")
    server = SOAPProxy("httpg://localhost:9900")
    
else: 
    # Default: use standard http
    pathserver = SOAPProxy("http://localhost:9900/pathtest")
    server = SOAPProxy("http://localhost:9900")

# Echo...

try:
    print server.echo("MOO")
except Exception, e:
    print "Caught exception: ", e
try:
    print pathserver.echo("MOO")
except Exception, e:
    print "Caught exception: ", e
    
# ...in an object
try:
    print server.echo_ino("moo")
except Exception, e:
    print "Caught exception: ", e
try:
    print pathserver.echo_ino("cow")
except Exception, e:
    print "Caught exception: ", e

# ...in an object in an object
try:
    print server.prop.echo2("moo")
except Exception, e:
    print "Caught exception: ", e

try:
    print pathserver.prop.echo2("cow")
except Exception, e:
    print "Caught exception: ", e

# ...with keyword arguments 
try:
    print server.echo_wkw(third = "three", first = "one", second = "two")
except Exception, e:
    print "Caught exception: ", e
try:
    print pathserver.echo_wkw(third = "three", first = "one", second = "two")
except Exception, e:
    print "Caught exception: ", e

# ...with a context object
try:
    print server.echo_wc("moo")
except Exception, e:
    print "Caught exception: ", e
try:
    print pathserver.echo_wc("cow")
except Exception, e:
    print "Caught exception: ", e

# ...with a header
hd = headerType(data = {"mystring": "Hello World"})
try:
    print server._hd(hd).echo_wc("moo")
except Exception, e:
    print "Caught exception: ", e
try:
    print pathserver._hd(hd).echo_wc("cow")
except Exception, e:
    print "Caught exception: ", e



#!/usr/bin/env python
#
# Copyright (c) 2001 actzero, inc. All rights reserved.

import sys
sys.path.insert(1, "..")

from SOAPpy import *

# Uncomment to see outgoing HTTP headers and SOAP and incoming
#Config.dumpSOAPIn = 1
#Config.dumpSOAPOut = 1
#Config.debug = 1

# specify name of authorization function
Config.authMethod = "_authorize"

# Set this to 0 to test authorization
allowAll = 1

# ask for returned SOAP responses to be converted to basic python types
Config.simplify_objects = 1

if Config.SSLserver:
    from M2Crypto import SSL
    
def _authorize(*args, **kw):
    global allowAll, Config

    if Config.debug:
        print "Authorize (function) called! (result = %d)" % allowAll
        print "Method: %s" % kw['method']
    
    if allowAll:
        return 1
    else:
        return 0

# Simple echo
def echo(s):
    global Config
    
    # Test of context retrieval
    ctx = Server.GetSOAPContext()
    if Config.debug:
        print "SOAP Context: ", ctx
        
    return s + s

# An echo class
class echoBuilder2:
    def echo2(self, val):
        return val * 3

# A class that has an instance variable which is an echo class
class echoBuilder:
    def __init__(self):
        self.prop = echoBuilder2()

    def echo_ino(self, val):
        return val + val
    def _authorize(self, *args, **kw):
        global allowAll, Config

        if Config.debug:
            print "Authorize (method) called with arguments:"
            print "*args=%s" % str(args)
            print "**kw =%s" % str(kw)
            print "Approved -> %d" % allowAll
        
        if allowAll:
            return 1
        else:
            return 0

# Echo with context
def echo_wc(s, _SOAPContext):
    global Config
    
    c = _SOAPContext

    sep = '-' * 72

    # The Context object has extra info about the call
    if Config.debug:
        print "-- XML", sep[7:]
        # The original XML request
        print c.xmldata     

        print "-- Header", sep[10:]
        # The SOAP Header or None if not present
        print c.header      

        if c.header:
            print "-- Header.mystring", sep[19:]
            # An element of the SOAP Header
            print c.header.mystring         

        print "-- Body", sep[8:]
        # The whole Body object
        print c.body        

        print "-- Peer", sep[8:]
        if not GSI:
            # The socket object, useful for
            print c.connection.getpeername()    
        else:
            # The socket object, useful for
            print c.connection.get_remote_address() 
            ctx = c.connection.get_security_context()
            print ctx.inquire()[0].display()

        print "-- SOAPAction", sep[14:]
        # The SOAPaction HTTP header
        print c.soapaction                  

        print "-- HTTP headers", sep[16:]
        # All the HTTP headers
        print c.httpheaders                 

    return s + s

# Echo with keyword arguments
def echo_wkw(**kw):
    return kw['first'] + kw['second'] + kw['third']

addr = ('localhost', 9900)
GSI = 0
SSL = 0
if len(sys.argv) > 1 and sys.argv[1] == '-s':
    SSL = 1
    if not Config.SSLserver:
        raise RuntimeError, \
            "this Python installation doesn't have OpenSSL and M2Crypto"
    ssl_context = SSL.Context()
    ssl_context.load_cert('validate/server.pem')
    server = SOAPServer(addr, ssl_context = ssl_context)
    prefix = 'https'
elif len(sys.argv) > 1 and sys.argv[1] == '-g':
    GSI = 1
    from SOAPpy.GSIServer import GSISOAPServer
    server = GSISOAPServer(addr)
    prefix = 'httpg'
else:
    server = SOAPServer(addr)
    prefix = 'http'

print "Server listening at: %s://%s:%d/" % (prefix, addr[0], addr[1])

# register the method
server.registerFunction(echo)
server.registerFunction(echo, path = "/pathtest")
server.registerFunction(_authorize)
server.registerFunction(_authorize, path = "/pathtest")

# Register a whole object
o = echoBuilder()
server.registerObject(o, path = "/pathtest")
server.registerObject(o)

# Register a function which gets called with the Context object
server.registerFunction(MethodSig(echo_wc, keywords = 0, context = 1),
                        path = "/pathtest")
server.registerFunction(MethodSig(echo_wc, keywords = 0, context = 1))

# Register a function that takes keywords
server.registerKWFunction(echo_wkw, path = "/pathtest")
server.registerKWFunction(echo_wkw)

# Start the server
try:
    server.serve_forever()
except KeyboardInterrupt:
    pass



#!/usr/bin/env python

import string
import cgi

ident = '$Id: interop2html.py,v 1.1.1.1 2001/06/27 21:36:14 cullman Exp $'

lines = open('output.txt').readlines()
#preserve the tally
tally = lines[-6:]
#whack the tally from lines
lines = lines[:-6]
table={}
for line in lines:
  if line[:3] == '   ' or line == '>\n' : continue
  line = line[:-1]  #delete end of line char
  row = [line[:line.find(': ')], line[line.find(': ')+2:]]  #split server name from rest of line
  restofrow = row[1].split(' ',3)  #break out method name, number, status code, status comment
  if len(restofrow) > 3:
    if restofrow[3].find('as expected') != -1:
      restofrow[2] = restofrow[2] + ' (as expected)'
    elif restofrow[3][:2] == '- ' :
      restofrow[3] = restofrow[3][2:] 
  try: table[row[0]].append([restofrow[0],restofrow[2:]])
  except KeyError: table[row[0]] = [[restofrow[0],restofrow[2:]]]

print "<html><body>"
print "<script>function popup(text) {"
print "text = '<html><head><title>Test Detail</title></head><body><p>' + text + '</p></body></html>';"
print "newWin=window.open('','win1','location=no,menubar=no,width=400,height=200');"
print "newWin.document.open();"
print "newWin.document.write(text);"
print "newWin.focus();  } </script>"
print "<br><table style='font-family: Arial; color: #cccccc'><tr><td colspan=2><font face=arial color=#cccccc><b>Summary</b></font></td></tr>"
for x in tally:
  z = x[:-1].split(":",1)
  print "<tr><td><font face=arial color=#cccccc>",z[0],"</font></td><td><font face=arial color=#cccccc>",z[1],"</font></td></tr>"
print "</table><br>"
c = 0
totalmethods = len(table[table.keys()[0]])
while c < totalmethods:
  print "<br><table width='95%' style='font-family: Arial'>"
  print "<tr><td width='27%' bgcolor='#cccccc'></td>"
  cols = [c, c + 1, c + 2]
  if c != 16:
    cols += [c + 3]
  for i in cols:
    try: header = table[table.keys()[0]][i][0]
    except: break
    print "<td width ='17%' align='center' bgcolor='#cccccc'><b>",header,"</b></td>"
  print "</tr>"
  l = table.keys()
  l.sort()
  for key in l:
    print "<tr><td bgcolor='#cccccc'>", key , "</td>"
    for i in cols:
      try: status = table[key][i][1][0]
      except: break
      if status.find("succeed") != -1:
        bgcolor = "#339900"
        status = "Pass"
      elif status.find("expected") != -1:
        bgcolor = "#FF9900"
        hreftitle = table[key][i][1][1].replace("'","") # remove apostrophes from title properties
        popuphtml = '"' + cgi.escape(cgi.escape(table[key][i][1][1]).replace("'","&#39;").replace('"',"&#34;")) + '"'
        status = "<a title='" + hreftitle + "' href='javascript:popup(" + popuphtml + ")'>Failed (expected)</a>"
      else:
        bgcolor = "#CC0000"
        hreftitle = table[key][i][1][1].replace("'","") # remove apostrophes from title properties
        popuphtml = '"' + cgi.escape(cgi.escape(table[key][i][1][1]).replace("'","&#39;").replace('"',"&#34;")) + '"'
        status = "<a title='" + hreftitle + "' href='javascript:popup(" + popuphtml + ")'>Failed</a>"
      print "<td align='center' bgcolor=" , bgcolor , ">" , status , "</td>"
    print "</tr>"
  print "</table>"
  c = c + len(cols)
print "</body></html>"



#!/usr/bin/env python

# This server validates as of 4/23/01 when run with UserLand's SOAP validator
# (http://validator.soapware.org/).

import getopt
import sys

sys.path.insert (1, '..')

from SOAPpy import SOAP

ident = '$Id: soapware.py,v 1.2 2003/03/08 05:10:01 warnes Exp $'

def whichToolkit ():
    return SOAP.SOAPUserAgent ()

def countTheEntities (s):
    counts = {'ctLeftAngleBrackets': 0, 'ctRightAngleBrackets': 0,
        'ctAmpersands': 0, 'ctApostrophes': 0, 'ctQuotes': 0}

    for i in s:
        if i == '<':
            counts['ctLeftAngleBrackets'] += 1
        elif i == '>':
            counts['ctRightAngleBrackets'] += 1
        elif i == '&':
            counts['ctAmpersands'] += 1
        elif i == "'":
            counts['ctApostrophes'] += 1
        elif i == '"':
            counts['ctQuotes'] += 1

    return counts

def easyStructTest (stooges):
    return stooges['larry'] + stooges['moe'] + stooges['curly']

def echoStructTest (myStruct):
    return myStruct

def manyTypesTest (num, bool, state, doub, dat, bin):
    return [num, SOAP.booleanType (bool), state, doub,
        SOAP.dateTimeType (dat), bin]

def moderateSizeArrayCheck (myArray):
    return myArray[0] + myArray[-1]

def nestedStructTest (myStruct):
    return easyStructTest (myStruct.year2000.month04.day01)

def simpleStructReturnTest (myNumber):
    return {'times10': myNumber * 10, 'times100': myNumber * 100,
        'times1000': myNumber * 1000}

namespace = 'http://www.soapware.org/'

DEFAULT_HOST    = 'localhost'
DEFAULT_PORT    = 8080

def usage (error = None):
    sys.stdout = sys.stderr

    if error != None:
        print error

    print """usage: %s [options]
  If a long option shows an argument is mandatory, it's mandatory for the
  equivalent short option also. The default (if any) is shown in brackets.

  -?, --help            display this usage
  -h, --host=HOST       use HOST in the address to listen on [%s]
  -p, --port=PORT       listen on PORT [%d]
""" % (sys.argv[0], DEFAULT_HOST, DEFAULT_PORT),

    sys.exit (0)

def main ():
    host = DEFAULT_HOST
    port = DEFAULT_PORT

    try:
        opts, args = getopt.getopt (sys.argv[1:], '?h:p:',
            ['help', 'host', 'port'])

        for opt, arg in opts:
            if opt in ('-?', '--help'):
                usage ()
            elif opt in ('-h', '--host'):
                host = arg
            elif opt in ('-p', '--port'):
                port = int (arg)
            else:
                raise AttributeError, \
                     "Recognized but unimplemented option `%s'" % opt
    except SystemExit:
        raise
    except:
        usage (sys.exc_info ()[1])

    server = SOAP.SOAPServer ((host, port))

    server.registerFunction (whichToolkit, namespace)
    server.registerFunction (countTheEntities)
    server.registerFunction (easyStructTest)
    server.registerFunction (echoStructTest)
    server.registerFunction (manyTypesTest)
    server.registerFunction (moderateSizeArrayCheck)
    server.registerFunction (nestedStructTest)
    server.registerFunction (simpleStructReturnTest)

    server.serve_forever()

if __name__ == '__main__':
    try:
        sys.exit (main ())
    except KeyboardInterrupt:
        sys.exit (0)



"""
################################################################################
#
# SOAPpy - Cayce Ullman       (cayce@actzero.com)
#          Brian Matthews     (blm@actzero.com)
#          Gregory Warnes     (gregory_r_warnes@groton.pfizer.com)
#          Christopher Blunck (blunck@gst.com)
#
################################################################################
# Copyright (c) 2003, Pfizer
# Copyright (c) 2001, Cayce Ullman.
# Copyright (c) 2001, Brian Matthews.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# Neither the name of actzero, inc. nor the names of its contributors may
# be used to endorse or promote products derived from this software without
# specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################
"""

ident = '$Id: Errors.py,v 1.4 2004/01/31 04:20:05 warnes Exp $'
from version import __version__

import exceptions

################################################################################
# Exceptions
################################################################################
class Error(exceptions.Exception):
    def __init__(self, msg):
        self.msg = msg
    def __str__(self):
        return "<Error : %s>" % self.msg
    __repr__ = __str__
    def __call__(self):
        return (msg,)

class RecursionError(Error):
    pass

class UnknownTypeError(Error):
    pass

class HTTPError(Error):
    # indicates an HTTP protocol error
    def __init__(self, code, msg):
        self.code = code
        self.msg  = msg
    def __str__(self):
        return "<HTTPError %s %s>" % (self.code, self.msg)
    __repr__ = __str__
    def __call___(self):
        return (self.code, self.msg, )

class UnderflowError(exceptions.ArithmeticError):
    pass

